helpMessage="""ComplexConstructor:

To build the macrocomplex fill in the main window requirements. As for running via 
command-line, a FASTA file with the sequences and a directory with the PDB files 
of the interacting are required. In addition, a name for the folder where the results 
will be stored is needed, after typing it you should confirm it. 

Note that, to select the PDB directory you have to enter in the desired directory 
and then select it. 

Furthermore, additional options can be set:

    In the main window you can specify if you want to create a log file where the 
    process of the execution will be displayed. 

    In the top menu, in the 'Options' dropdown there is the 'Add Stoichiometry' 
    option. You can upload a file with a determined stoichiometry to be applied 
    to the macrocomplex. The format of this file has to be the ID of the sequence, 
    concordant with the one in the FASTA file, followed by ':' and a number. This 
    number will be the number of times the corresponging sequence will be in the 
    final complex."""